#[allow(unused)]
const TESTFILES_DIR: &str = "testfiles";

#[allow(unused)]
const DIGTIAL_IDE_TEST: &str = "/home/dide/project/Digital-Test/Digital-IDE-test/user";

#[allow(unused)]
macro_rules! unwrap_result {
    ($expr:expr) => {
        match $expr {
            Ok(e) => e,
            Err(_) => return
        }
    };
}

#[allow(unused)]
macro_rules! unwrap_option {
    ($expr:expr) => {
        match $expr {
            Some(e) => e,
            None => return
        }
    };
}

#[cfg(test)]
mod test_fast {
    use std::{fs, path::Path};
    use crate::core::sv_parser::sv_parser;
    use super::*;

    #[test]
    fn test_testfiles() {
        let entries = unwrap_result!(fs::read_dir(TESTFILES_DIR));
        for entry in entries {
            let entry = unwrap_result!(entry);
            let file_path = entry.path();
            if file_path.is_file() {
                let extension = unwrap_option!(file_path.extension());
                let file_path = unwrap_option!(file_path.to_str());
                if extension == "v" || extension == "sv" {
                    println!("Test file: {:?}", file_path);
                    let _ = sv_parser(file_path);
                }
            }
        }
    }

    #[test]
    fn test_digital_ide_test() {
        // 判断路径是否存在且为文件夹
        let path = Path::new(DIGTIAL_IDE_TEST);
        if path.exists() && path.is_dir() {
            // 递归遍历文件夹
            if let Err(e) = traverse_directory(path) {
                eprintln!("Error: {}", e);
            }
        } else {
            eprintln!("Path does not exist or is not a directory");
        }
    }

    fn traverse_directory(dir: &Path) -> Result<(), Box<dyn std::error::Error>> {
        if dir.is_dir() {
            for entry in fs::read_dir(dir)? {
                let entry = entry?;
                let path = entry.path();
    
                if path.is_dir() {
                    // 递归遍历子文件夹
                    traverse_directory(&path)?;
                } else if path.is_file() {
                    // 检查文件扩展名
                    if let Some(ext) = path.extension() {
                        if ext == "v" || ext == "sv" {
                            println!("Test file: {:?}", path);
                            // TODO: Check Stack Overflow tests
                            if path.to_str().unwrap() == "/home/dide/project/Digital-Test/Digital-IDE-test/user/src/svlog/tools/ivtest/comp1001.sv" {
                                continue;
                            }
                            if path.to_str().unwrap() == "/home/dide/project/Digital-Test/Digital-IDE-test/user/src/svlog/tools/ivtest/comp1000.sv" {
                                continue;
                            }
                            if path.to_str().unwrap() == "/home/dide/project/Digital-Test/Digital-IDE-test/user/src/svlog/tools/ivtest/br_gh330.sv" {
                                continue;
                            }
                            if path.to_str().unwrap() == "/home/dide/project/Digital-Test/Digital-IDE-test/user/src/svlog/tools/hdlconv/pri_encoder_using_assign.sv" {
                                continue;
                            }
                            let file_path = path.to_str().unwrap();
                            let _ = sv_parser(file_path);
                        }
                    }
                }
            }
        }
        Ok(())
    }
}

#[cfg(test)]
mod test_scope_tree {
    use std::{collections::HashMap, fs, path::{Path, PathBuf}};
    use crate::definition::{get_scopes, GenericScope};
    use sv_parser::parse_sv;
    use tower_lsp::lsp_types::Url;

    use super::*;

    fn get_scope_tree(file_path: &str) -> Option<GenericScope> {
        let defines = HashMap::new();
        let includes: Vec<PathBuf> = Vec::new();

        let result = parse_sv(file_path, &defines, &includes, true, true);
        if let Ok((syntax_tree, _)) = result {
            let file_url = format!("file://{}", file_path);
            let uri = Url::parse(&file_url);
            if let Ok(uri) = uri {
                return get_scopes(&syntax_tree, &uri);
            } else {
                eprintln!("error happen when unwrap uri: {:?}", uri);
                return None;
            }
        }
        None
    }

    #[test]
    fn test_testfiles() {
        let entries = unwrap_result!(fs::read_dir(TESTFILES_DIR));
        for entry in entries {
            let entry = unwrap_result!(entry);
            let file_path = entry.path();
            if file_path.is_file() {
                let extension = unwrap_option!(file_path.extension());
                let file_path = unwrap_option!(file_path.to_str());
                if extension == "v" || extension == "sv" {
                    println!("Test file: {:?}", file_path);
                    let _ = get_scope_tree(file_path);
                }
            }
        }
    }

    #[test]
    fn test_digital_ide_test() {
        // 判断路径是否存在且为文件夹
        let path = Path::new(DIGTIAL_IDE_TEST);
        if path.exists() && path.is_dir() {
            // 递归遍历文件夹
            if let Err(e) = traverse_directory(path) {
                eprintln!("Error: {}", e);
            }
        } else {
            eprintln!("Path does not exist or is not a directory");
        }
    }

    fn traverse_directory(dir: &Path) -> Result<(), Box<dyn std::error::Error>> {
        if dir.is_dir() {
            for entry in fs::read_dir(dir)? {
                let entry = entry?;
                let path = entry.path();
    
                if path.is_dir() {
                    // 递归遍历子文件夹
                    traverse_directory(&path)?;
                } else if path.is_file() {
                    // 检查文件扩展名
                    if let Some(ext) = path.extension() {
                        if ext == "v" || ext == "sv" {
                            println!("Test file: {:?}", path);
                            // TODO: Check Stack Overflow tests
                            if path.to_str().unwrap() == "/home/dide/project/Digital-Test/Digital-IDE-test/user/src/svlog/tools/ivtest/comp1001.sv" {
                                continue;
                            }
                            if path.to_str().unwrap() == "/home/dide/project/Digital-Test/Digital-IDE-test/user/src/svlog/tools/ivtest/comp1000.sv" {
                                continue;
                            }
                            if path.to_str().unwrap() == "/home/dide/project/Digital-Test/Digital-IDE-test/user/src/svlog/tools/ivtest/br_gh330.sv" {
                                continue;
                            }
                            if path.to_str().unwrap() == "/home/dide/project/Digital-Test/Digital-IDE-test/user/src/svlog/tools/hdlconv/pri_encoder_using_assign.sv" {
                                continue;
                            }
                            let file_path = path.to_str().unwrap();
                            let _ = get_scope_tree(file_path);
                        }
                    }
                }
            }
        }
        Ok(())
    }
}